import { world, system } from "@minecraft/server";
import { ModalFormData } from "@minecraft/server-ui";

// Variabel untuk menyimpan status toggle dan dropdown
let stayCamStatus = false;
let selectedCamMode = 0;  // Nilai default untuk First Person

// Membuat form yang berisi dropdown untuk memilih mode kamera, toggle untuk Stay CAM
function showForm(player) {
  const form = new ModalFormData()
    .title("monoCAM Camera Modes")
    .dropdown("Select Camera Mode", ["First Person", "Third Person (Back)", "Third Person (Front)"], selectedCamMode)  // Menggunakan nilai tersimpan untuk dropdown
    .toggle("Stay CAM OFF/ON", stayCamStatus);  // Menggunakan nilai tersimpan untuk Stay CAM

  form.show(player).then((response) => {
    // Jika form ditutup tanpa memilih apapun
    if (response.canceled) {
      return player.runCommandAsync("title @s actionbar You canceled the form");
    }

    selectedCamMode = response.formValues[0];  // Mengambil dan menyimpan nilai dari dropdown
    stayCamStatus = response.formValues[1];  // Mengambil nilai dari toggle untuk Stay CAM dan menyimpannya

    // Gabungkan semua logika dalam satu chain untuk menghindari duplikasi
    player.runCommandAsync("playsound note.xylophone @s")
      .then(() => {
        // Mengatur mode kamera berdasarkan pilihan dropdown
        switch (selectedCamMode) {
          case 0:
            return player.runCommandAsync("camera @s clear");
          case 1:
            return player.runCommandAsync("camera @s set minecraft:third_person");
          case 2:
            return player.runCommandAsync("camera @s set minecraft:third_person_front");
        }
      })
      .then(() => {
        // Mengatur Stay CAM ON/OFF berdasarkan toggle
        return player.runCommandAsync(stayCamStatus
          ? "tag @s add camposit"
          : "tag @s remove camposit");
      })
      .then(() => {
        return player.runCommandAsync(stayCamStatus
          ? "camera @s set minecraft:free pos ~ ~1.5 ~ rot ~ ~"
          : "tag @s remove camfollow");
      })
      .catch((e) => {
        player.runCommandAsync("title @s actionbar Failed to set camera or Stay CAM mode");
      });
  }).catch((error) => {
    player.runCommandAsync("title @s actionbar Failed to show form");
  });
}

// Menangani penggunaan item monocam
world.beforeEvents.itemUse.subscribe((eventData) => {
  if (eventData.itemStack?.typeId === "md:monocam") {
    let player = eventData.source;
    system.run(() => {
      showForm(player);
    });
  }
});