import { world } from '@minecraft/server';

// Fungsi untuk memutar suara dan menampilkan partikel
function playEffectWithParticles(block) {
    block.dimension.playSound("dig.powder_snow", block.location);
    block.dimension.runCommandAsync(`particle minecraft:cauldron_explosion_emitter ${block.location.x} ${block.location.y + 0.5} ${block.location.z}`);
}

// Fungsi untuk pengecekan apakah pemain memegang tool tertentu
function isHoldingTool(player, toolType) {
    const selectedItem = player.getComponent('equippable')?.getEquipment('Mainhand');
    return selectedItem?.typeId === toolType || false;
}

// Fungsi untuk update state block
function updateBlockState(block, stateKey, increment, maxState) {
    const currentState = block.permutation?.getState(stateKey) ?? 0;
    const newState = (currentState + increment) % maxState;
    block.setPermutation(block.permutation.withState(stateKey, newState));
    playEffectWithParticles(block);
}

// Gabungkan semua komponen ke dalam satu subscribe
world.beforeEvents.worldInitialize.subscribe(eventData => {
    const components = [
        { name: 'md:rotate', stateKey: 'md:rotate', maxState: 3 },
        { name: 'md:push_0', stateKey: 'md:inout', maxState: 2 },
        { name: 'md:push_1', stateKey: 'md:inout', maxState: 4 }
    ];

    components.forEach(({ name, stateKey, maxState }) => {
        eventData.blockComponentRegistry.registerCustomComponent(name, {
            onPlayerInteract(e) {
                if (isHoldingTool(e.player, 'md:md_tool')) {
                    updateBlockState(e.block, stateKey, 1, maxState);
                }
            }
        });
    });
});
