import { BlockStates, world, system } from "@minecraft/server";
const record = {};

// Fungsi untuk menampilkan pesan
function message(msg, player) {
    return player.runCommandAsync(`titleraw @s actionbar {"rawtext":[{"text":${JSON.stringify(msg)}}]}`);
}

// Event untuk mendeteksi ketika entity memukul block
world.afterEvents.entityHitBlock.subscribe((ev) => {
    const player = ev.damagingEntity;
    if (player?.typeId !== "minecraft:player" || !player.hasTag("debug")) return;
    
    const selectedItem = player.getComponent("minecraft:inventory").container.getItem(player.selectedSlotIndex);
    if (selectedItem?.typeId === "md:debug_stick") {
        changeSelectStates(player, ev.hitBlock);
    }
});

// Event untuk menggunakan item pada block
world.beforeEvents.itemUseOn.subscribe((ev) => {
    const player = ev.source;
    if (player.typeId !== "minecraft:player" || ev.itemStack.typeId !== "md:debug_stick" || !player.hasTag("debug")) return;

    ev.cancel = true;
    if (player.isSneaking) {
        blockInfo(player, ev.block);
    } else {
        changeBlockStates(player, ev.block);
    }
});

// Fungsi untuk mengubah state yang dipilih
function changeSelectStates(player, block) {
    const permutation = block.permutation;
    const states = permutation.getAllStates();
    const names = Object.keys(states);
    
    if (!names.length && !block.type.canBeWaterlogged) {
        return message(`${block.typeId} don't have states`, player);
    }
    
    let prop = names[names.indexOf(record[player.id]) + 1] || (block.type.canBeWaterlogged ? "waterlogged" : names[0]);
    let val = prop === "waterlogged" ? block.isWaterlogged : states[prop];
    
    record[player.id] = prop;
    message(`selected = ${prop} (${val})`, player);
}

// Fungsi untuk mengubah state block
function changeBlockStates(player, block) {
    const permutation = block.permutation;
    const states = permutation.getAllStates();
    const names = Object.keys(states);

    if (!names.length && !block.type.canBeWaterlogged) {
        return message(`${block.typeId} don't have states`, player);
    }

    let prop = record[player.id];
    let val; // Inisialisasi val di sini

    // Jika prop adalah "waterlogged" dan block bisa berair
    if (prop == "waterlogged") {
        if (block.type.canBeWaterlogged) {
            val = !block.isWaterlogged;
            system.run(() => {
                block.setWaterlogged(val);
            });
        } else {
            // Jika block tidak bisa berair, set prop ke state pertama yang ada
            prop = names[0];
            val = states[prop];
        }
    } else {
        if (!names.includes(prop)) {
            prop = names[0]; // Set prop ke state pertama jika prop tidak valid
        }

        // Ambil valid values dan lakukan pergantian
        const valids = BlockStates.get(prop).validValues;
        val = valids[valids.indexOf(states[prop]) + 1];
        if (typeof val === "undefined") {
            val = valids[0];
        }

        system.run(() => {
            block.setPermutation(permutation.withState(prop, val));
        });
    }

    // Simpan state yang terakhir diubah
    record[player.id] = prop;
    message(`change ${prop} to ${val}`, player);
}


// Fungsi untuk mendapatkan informasi block
function blockInfo(player, block) {
    let info = `§l${block.typeId}§r\nPosition: ${block.x}, ${block.y}, ${block.z}`;
    Object.entries(block.permutation.getAllStates()).forEach(([k, v]) => {
        let color = typeof v === "string" ? "§6" : typeof v === "number" ? "§e" : "§b";
        info += `\n§o§7${k}§r§8: ${color}${v}`;
    });
    block.getTags().forEach(tag => info += `\n§dTag§r: ${tag}`);
    message(info, player);
}

// CODE COPY TOOL
const cooldowns = new Map();
const cooldownDuration = 15; // Cooldown dalam ticks (0.75 detik)
const cooldownMessages = new Set(); // Menyimpan player yang sudah diberi pesan cooldown

// Fungsi untuk memetakan block ke item counterpart yang valid
function getItemFromBlock(blockTypeId) {
    const blockToItemMap = {
        "minecraft:oak_standing_sign": "minecraft:oak_sign",
        "minecraft:birch_standing_sign": "minecraft:birch_sign",
        "minecraft:birch_wall_sign": "minecraft:birch_sign",
        "minecraft:darkoak_standing_sign": "minecraft:dark_oak_sign",
        "minecraft:darkoak_wall_sign": "minecraft:dark_oak_sign",
        "minecraft:spruce_wall_sign": "minecraft:spruce_sign",
        "minecraft:spruce_standing_sign": "minecraft:spruce_sign",
        "minecraft:warped_standing_sign": "minecraft:warped_sign",
        "minecraft:crimson_standing_sign": "minecraft:crimson_sign",
        "minecraft:bamboo_standing_sign": "minecraft:bamboo_sign",
        "minecraft:cherry_standing_sign": "minecraft:cherry_sign",
        "minecraft:acacia_standing_sign": "minecraft:acacia_sign",
        "minecraft:jungle_standing_sign": "minecraft:jungle_sign",
        "minecraft:mangrove_standing_sign": "minecraft:mangrove_sign",
        "minecraft:standing_sign": "minecraft:oak_sign", // Default to oak_sign
        "minecraft:standing_banner": "minecraft:banner",
        "minecraft:wall_banner": "minecraft:banner",
        // Tambahkan block lain jika perlu
    };

    return blockToItemMap[blockTypeId] || blockTypeId; // Kembalikan item yang valid, atau blockTypeId aslinya
}

// Event untuk menggunakan item pada block (Copy Tool)
world.beforeEvents.itemUseOn.subscribe((ev) => {
    const player = ev.source;
    if (player.typeId !== "minecraft:player" || ev.itemStack.typeId !== "md:md_scanner") return;

    ev.cancel = true;
    const block = ev.block;
    const playerId = player.id;

    // Dapatkan item counterpart dari block
    const itemTypeId = getItemFromBlock(block.typeId);

    // Menampilkan nama block di chat
    const blockName = block.typeId.replace("minecraft:", "").replace(/_/g, " "); // Format nama block
    const formattedBlockName = blockName.charAt(0).toUpperCase() + blockName.slice(1); // Kapitalisasi huruf pertama

    if (!cooldowns.has(playerId)) {
        if (player.isSneaking) {
            // Jika sneak, berikan item ke player
            player.runCommandAsync(`give @s ${itemTypeId} 1`);
            player.runCommandAsync(`title @s actionbar "Give metode"`);
            player.sendMessage(`§eYou copied: ${formattedBlockName}`);
        } else {
            // Jika tidak sneak, replace item di hotbar
            player.runCommandAsync(`replaceitem entity @s slot.hotbar 0 ${itemTypeId}`);
            player.runCommandAsync(`title @s actionbar "Replace metode"`);
            player.sendMessage(`§eYou replaced with: ${formattedBlockName}`);
        }
        player.runCommandAsync("playsound note.flute @s");
        cooldowns.set(playerId, cooldownDuration); // Set cooldown setelah tindakan
        cooldownMessages.delete(playerId); // Reset pesan cooldown
    } else {
        // Hanya kirim pesan cooldown sekali
        if (!cooldownMessages.has(playerId)) {
            player.runCommandAsync(`title @s actionbar "Cooldown in progress..."`);
            cooldownMessages.add(playerId); // Tandai bahwa pesan cooldown sudah dikirim
        }
    }
});

// Mengurangi cooldown setiap beberapa tick (misalnya setiap 5 tick)
system.runInterval(() => {
    cooldowns.forEach((value, playerId) => {
        if (value > 0) {
            cooldowns.set(playerId, value - 5); // Kurangi setiap 5 tick
        } else {
            cooldowns.delete(playerId); // Hapus cooldown setelah selesai
            cooldownMessages.delete(playerId); // Hapus tanda pesan cooldown
        }
    });
}, 5);  // Mengurangi setiap 5 tick
